#include <OneWire.h>
#include <DallasTemperature.h>
#include <SPI.h>
#include <SD.h>

// SD-Card
File myFile;
unsigned long oldStartTime = 0;
unsigned long startTime = 0;
int MessungNum = 0;

// EC Sensor
int R1=1000;
int Ra=25;
int ECPin=A1;
int ECGround=A5;
int ECPower=A4;

float PPMconversion=0.64;
float TemperatureCoef = 0.019;
float K=2.88;

// Temperature
#define ONE_WIRE_BUS 2          // Data wire For Temp Probe is plugged into pin 10 on the Arduino
const int TempProbePossitive =8;  //Temp Probe power connected to pin 9
const int TempProbeNegative=9;    //Temp Probe Negative connected to pin 8

OneWire oneWire(ONE_WIRE_BUS);// Setup a oneWire instance to communicate with any OneWire devices
DallasTemperature sensors(&oneWire);// Pass our oneWire reference to Dallas Temperature.

float Temperature=10;
float EC=0;
float EC25 =0;
int ppm =0;

float raw= 0;
float Vin= 5;
float Vdrop= 0;
float Rc= 0;
float buffer=0;
 
float Celsius = 0;
float Fahrenheit = 0;

// Waterlevel
const int waterAnalogPin = A2;
const int waterSensorPowerPin = 4;

//pH-Value
#define SensorPin A5          // Der pH-Meter Analogausgang ist mit dem Analogeingang des Arduino verbunden
unsigned long int avgValue;  // Durchschnittswert des Sensors
float b;
int buf[10], temp;
float phValue = 0;

void setup() {
  Serial.begin(9600);

  // SD
  Serial.print("Initializing SD card...");
  if (!SD.begin(5)) {
    Serial.println("initialization failed!");
    //while (1);
  }
  Serial.println("initialization done.");

  // Pin Settings for Temp
  pinMode(TempProbeNegative , OUTPUT ); //seting ground pin as output for tmp probe
  digitalWrite(TempProbeNegative , LOW );//Seting it to ground so it can sink current
  pinMode(TempProbePossitive , OUTPUT );//ditto but for positive
  digitalWrite(TempProbePossitive , HIGH );

  // Pin Settings for EC
  pinMode(ECPin,INPUT);
  pinMode(ECPower,OUTPUT);//Setting pin for sourcing current
  pinMode(ECGround,OUTPUT);//setting pin for sinking current
  digitalWrite(ECGround,LOW);//We can leave the ground connected permanantly
 
  delay(100);// gives sensor time to settle
  sensors.begin();
  delay(100);
  
  R1=(R1+Ra);// Taking into acount Powering Pin Resitance

  // keeping track of Time
  oldStartTime = startTime;
  startTime = millis(); // Startzeitpunkt speichern
}

void loop() {
  // Take initial Probes
  if(MessungNum == 0){
  WriteElapsedTime();

  // pH Value
  GetPHValue(); // Value in phValue

  // EC and Temp Value
  GetEC();

  // Write Values on SD-Card
  WriteAllValuesOnSD();

  // In Case Computer is connected
  WriteValuesOnConsole();
  }

  // Temp Value every 3h
  GetTemp();
  WriteTempOnSD();
  Serial.print(Temperature); // In Case Computer is Connected

  delay(10800000); // wait 3h
}

void WriteElapsedTime(){
  unsigned long elapsedTime = oldStartTime - startTime;

  myFile = SD.open("wasser1.txt", FILE_WRITE);
  if (myFile) {
    //Serial.print("Writing to test.txt...");
    myFile.print("Vergangene Zeit zwischen letzer vollständigen Probenahme und neuer vollständigen Probenahme: ");
    myFile.print(elapsedTime / 1000 / 60 /60 );
    myFile.println(" Stunden");
    myFile.println();
    // close the file:
    myFile.close();
    Serial.println(elapsedTime);
    startTime = millis();
  }
  else {
    // if the file didn't open, print an error:
    Serial.println("Error opening file");
  }
}

void WriteTempOnSD(){
  unsigned long currentTime = millis();
  unsigned long elapsedTime = currentTime - startTime;

  myFile = SD.open("wasser1.txt", FILE_WRITE);
  if (myFile) {
    //Serial.print("Writing to test.txt...");
    myFile.print("Zeit: ");
    myFile.print(elapsedTime / 1000 / 60 /60 );
    myFile.println(" Stunden");
    myFile.print("Messung: ");
    myFile.println(MessungNum);
    myFile.print("Temperatur: ");
    myFile.println(Temperature);
    myFile.println();
    // close the file:
    myFile.close();
    MessungNum++;
  }
  else {
    // if the file didn't open, print an error:
    Serial.println("Error opening file");
  }
}

void WriteValuesOnConsole(){
  Serial.print("PH-Wert");
  Serial.print(phValue);
  Serial.print(" Rc: ");
  Serial.print(Rc);
  Serial.print(" EC: ");
  Serial.print(EC25);
  Serial.print(" Simens  ");
  Serial.print(ppm);
  Serial.print(" ppm  ");
  Serial.print(Temperature);
  Serial.println(" *C ");

}

void WriteAllValuesOnSD(){
  unsigned long currentTime = millis();
  unsigned long elapsedTime = currentTime - startTime;

  myFile = SD.open("wasser1.txt", FILE_WRITE);
  if (myFile) {
    //Serial.print("Writing to test.txt...");
    myFile.print("Zeit: ");
    myFile.print(elapsedTime / 1000 / 60 /60 );
    myFile.println(" Stunden");
    myFile.print("Messung: ");
    myFile.println(MessungNum);
    myFile.print("Temperatur: ");
    myFile.println(Temperature);
    myFile.print("PH-Wert");
    myFile.print(phValue);
    myFile.print("Rc: ");
    myFile.println(Rc);
    myFile.print("EC: ");
    myFile.println(EC25);
    myFile.print("ppm: ");
    myFile.println(ppm);
    myFile.println();
    // close the file:
    myFile.close();
    MessungNum++;
  }
  else {
    // if the file didn't open, print an error:
    Serial.println("Error opening file");
  }
}

void GetTemp(){
  sensors.requestTemperatures();// Send the command to get temperatures
  Temperature=sensors.getTempCByIndex(0); //Stores Value in Variable
}

void GetEC(){
  //*********Reading Temperature Of Solution *******************//
  GetTemp();

  //************Estimates Resistance of Liquid ****************//
  digitalWrite(ECPower,HIGH);
  raw= analogRead(ECPin);
  raw= analogRead(ECPin);// This is not a mistake, First reading will be low beause if charged a capacitor
  digitalWrite(ECPower,LOW);
  
  //***************** Converts to EC **************************//
  Vdrop= (Vin*raw)/1024.0;
  Rc=(Vdrop*R1)/(Vin-Vdrop);
  Rc=Rc-Ra; //acounting for Digital Pin Resitance
  EC = 1000/(Rc*K);
  
  //*************Compensating For Temperature********************//
  EC25  =  EC/ (1+ TemperatureCoef*(Temperature-25.0));
  ppm=(EC25)*(PPMconversion*1000);
}

void GetPHValue(){
  for(int i = 0; i < 10; i++)       // 10 Messwerte für eine gleichmäßige Messung
  { 
    buf[i] = analogRead(SensorPin);
    delay(10);
  }
  
  for(int i = 0; i < 9; i++)        // Sortiere die Werte von klein nach groß
  {
    for(int j = i + 1; j < 10; j++)
    {
      if(buf[i] > buf[j])
      {
        temp = buf[i];
        buf[i] = buf[j];
        buf[j] = temp;
      }
    }
  }

  avgValue = 0;
  for(int i = 2; i < 8; i++)                      // Durchschnittswert der 6 mittleren Werte
    avgValue += buf[i];

  phValue = (float)avgValue * 5.0 / 1024 / 6; // Analog in Millivolt umrechnen
  phValue = 13.0682 * phValue;                           // Millivolt in pH-Wert umrechnen

  // Kalibrierungskorrektur basierend auf der gemessenen Abweichung
  float calibration_offset = -18.0038; // Hier 0.3 als Beispielwert, du musst dies entsprechend deiner Messabweichung anpassen
  phValue += calibration_offset;

  //Serial.print("pH: ");  
  //Serial.print(phValue, 2);
  //Serial.println(" ");
  digitalWrite(7, HIGH);       
  delay(800);
  digitalWrite(7, LOW); 
}